==> klayout-0.17.1.tar.gz: klayout-0.17.1/src/layOASISWriterPlugin.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "dbOASIS.h"
#include "dbOASISWriter.h"
#include "dbSaveLayoutOptions.h"
#include "layMainWindow.h"
#include "layStream.h"

#include "OASISWriterOptionPage.h"

#include <QtGui/QFrame>

namespace lay
{

const std::string cfg_oasis_compression ("oasis-compression");

// ---------------------------------------------------------------
//  OASISWriterOptionPage definition and implementation

class OASISWriterOptionPage 
  : public ConfigPage
{
public:
  OASISWriterOptionPage (QWidget *parent);
  ~OASISWriterOptionPage ();

  void setup (Plugin *root);
  void commit (Plugin *root);

private:
  Ui::OASISWriterOptionPage *mp_ui;
};

OASISWriterOptionPage::OASISWriterOptionPage (QWidget *parent)
  : ConfigPage (parent)
{
  mp_ui = new Ui::OASISWriterOptionPage ();
  mp_ui->setupUi (this);
}

OASISWriterOptionPage::~OASISWriterOptionPage ()
{
  delete mp_ui;
}

void 
OASISWriterOptionPage::setup (Plugin *root)
{
  int compr = 2;
  root->config_get (cfg_oasis_compression, compr);
  mp_ui->compression_slider->setValue (compr);
}

void 
OASISWriterOptionPage::commit (Plugin *root)
{
  root->config_set (cfg_oasis_compression, mp_ui->compression_slider->value ());
}

// ---------------------------------------------------------------
//  OASISWriterPluginDeclaration definition and implementation

class OASISWriterPluginDeclaration
  : public StreamWriterPluginDeclaration
{
public:
  OASISWriterPluginDeclaration () 
    : StreamWriterPluginDeclaration ("OASIS" /*same than delivered by the stream format decl*/)
  {
    // .. nothing yet ..
  }

  ConfigPage *config_page (QWidget *parent, std::string &title) const
  {
    title = ""; // not to be put into the global setup dialog
    return new OASISWriterOptionPage (parent);
  }

  void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    db::OASISWriterOptions m_options;
    options.push_back (std::make_pair (cfg_oasis_compression, tl::to_string (m_options.compression_level)));
  }

  db::FormatSpecificWriterOptions *create_specific_options (lay::PluginRoot *config_root) const
  {
    db::OASISWriterOptions *options = new db::OASISWriterOptions ();

    try {
      config_root->config_get (cfg_oasis_compression, options->compression_level);
    } catch (...) {
      delete options;
      throw;
    }

    return options;
  }
};

static tl::Registrar<lay::PluginDeclaration>::Class plugin_decl (new lay::OASISWriterPluginDeclaration (), 10000, "OASISWriter");

// ---------------------------------------------------------------
//  gsi Implementation of specific methods

static void get_oasis_options (const db::SaveLayoutOptions *options, db::OASISWriterOptions &oasis_options)
{
  const db::OASISWriterOptions *current_oasis_options = dynamic_cast <const db::OASISWriterOptions *> (options->get_specific_options ());
  if (current_oasis_options) {
    oasis_options = *current_oasis_options;
  }
}

static void set_oasis_compression (db::SaveLayoutOptions *options, int comp)
{
  db::OASISWriterOptions oasis_options;
  get_oasis_options (options, oasis_options);
  oasis_options.compression_level = comp;
  options->set_specific_options (oasis_options);
}

static int get_oasis_compression (const db::SaveLayoutOptions *options)
{
  db::OASISWriterOptions oasis_options;
  get_oasis_options (options, oasis_options);
  return oasis_options.compression_level;
}

#ifdef HAVE_RUBY

//  extend lay::SaveLayoutOptions with the OASIS options 
static
gsi::Class<db::SaveLayoutOptions> oasis_writer_options ("SaveLayoutOptions",
  gsi::method_ext ("oasis_compression_level=", &set_oasis_compression,
    "@brief Set the OASIS compression level\n"
    "The OASIS compression level is an integer number between 0 and 10. 0 basically is no compression, "
    "1 produces shape arrays in a simple fashion. 2 and higher compression levels will use a more elaborate "
    "algorithm to find shape arrays which uses 2nd and futher neighbor distances. The higher the level, the "
    "higher the memory requirements and run times.\n"
    "Setting this property clears all format specific options for other formats such as GDS.\n"
  ) +
  gsi::method_ext ("oasis_compression_level", &get_oasis_compression,
    "@brief Get the OASIS compression level\n"
    "See \\oasis_compression_level= method for a description of the OASIS compression level."
  ),
  ""
);

#endif

}

==> klayout-0.19.3.tar.gz: klayout-0.19.3/src/layOASISWriterPlugin.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "dbOASIS.h"
#include "dbOASISWriter.h"
#include "dbSaveLayoutOptions.h"
#include "layMainWindow.h"
#include "layStream.h"

#include "OASISWriterOptionPage.h"

#include <QtGui/QFrame>

namespace lay
{

const std::string cfg_oasis_compression ("oasis-compression");

// ---------------------------------------------------------------
//  OASISWriterOptionPage definition and implementation

class OASISWriterOptionPage 
  : public StreamWriterOptionsPage
{
public:
  OASISWriterOptionPage (QWidget *parent);
  ~OASISWriterOptionPage ();

  void setup (const db::FormatSpecificWriterOptions *options);
  void commit (db::FormatSpecificWriterOptions *options);

private:
  Ui::OASISWriterOptionPage *mp_ui;
};

OASISWriterOptionPage::OASISWriterOptionPage (QWidget *parent)
  : StreamWriterOptionsPage (parent)
{
  mp_ui = new Ui::OASISWriterOptionPage ();
  mp_ui->setupUi (this);
}

OASISWriterOptionPage::~OASISWriterOptionPage ()
{
  delete mp_ui;
}

void 
OASISWriterOptionPage::setup (const db::FormatSpecificWriterOptions *o)
{
  const db::OASISWriterOptions *options = dynamic_cast<const db::OASISWriterOptions *> (o);
  if (options) {
    mp_ui->compression_slider->setValue (options->compression_level);
  }
}

void 
OASISWriterOptionPage::commit (db::FormatSpecificWriterOptions *o)
{
  db::OASISWriterOptions *options = dynamic_cast<db::OASISWriterOptions *> (o);
  if (options) {
    options->compression_level = mp_ui->compression_slider->value ();
  }
}

// ---------------------------------------------------------------
//  OASISWriterPluginDeclaration definition and implementation

class OASISWriterPluginDeclaration
  : public StreamWriterPluginDeclaration
{
public:
  OASISWriterPluginDeclaration () 
    : StreamWriterPluginDeclaration ("OASIS" /*same than delivered by the stream format decl*/)
  {
    // .. nothing yet ..
  }

  StreamWriterOptionsPage *format_specific_options_page (QWidget *parent) const
  {
    return new OASISWriterOptionPage (parent);
  }

  void get_options (std::vector < std::pair<std::string, std::string> > &options) const
  {
    db::OASISWriterOptions m_options;
    options.push_back (std::make_pair (cfg_oasis_compression, tl::to_string (m_options.compression_level)));
  }

  db::FormatSpecificWriterOptions *create_specific_options (lay::PluginRoot *config_root) const
  {
    db::OASISWriterOptions *options = new db::OASISWriterOptions ();

    //  no initialization if config_root is not set.
    if (config_root) {
      try {
        config_root->config_get (cfg_oasis_compression, options->compression_level);
      } catch (...) {
        delete options;
        throw;
      }
    }

    return options;
  }

  void commit_to_config (const db::FormatSpecificWriterOptions *o, lay::PluginRoot *config_root) const 
  {
    const db::OASISWriterOptions *options = dynamic_cast<const db::OASISWriterOptions *> (o);
    if (options) {
      config_root->config_set (cfg_oasis_compression, options->compression_level);
    }
  }
};

static tl::Registrar<lay::PluginDeclaration>::Class plugin_decl (new lay::OASISWriterPluginDeclaration (), 10000, "OASISWriter");

// ---------------------------------------------------------------
//  gsi Implementation of specific methods

static void get_oasis_options (const db::SaveLayoutOptions *options, db::OASISWriterOptions &oasis_options)
{
  const db::OASISWriterOptions *current_oasis_options = dynamic_cast <const db::OASISWriterOptions *> (options->get_specific_options ());
  if (current_oasis_options) {
    oasis_options = *current_oasis_options;
  }
}

static void set_oasis_compression (db::SaveLayoutOptions *options, int comp)
{
  db::OASISWriterOptions oasis_options;
  get_oasis_options (options, oasis_options);
  oasis_options.compression_level = comp;
  options->set_specific_options (oasis_options);
}

static int get_oasis_compression (const db::SaveLayoutOptions *options)
{
  db::OASISWriterOptions oasis_options;
  get_oasis_options (options, oasis_options);
  return oasis_options.compression_level;
}

#ifdef HAVE_RUBY

//  extend lay::SaveLayoutOptions with the OASIS options 
static
gsi::Class<db::SaveLayoutOptions> oasis_writer_options ("SaveLayoutOptions",
  gsi::method_ext ("oasis_compression_level=", &set_oasis_compression,
    "@brief Set the OASIS compression level\n"
    "The OASIS compression level is an integer number between 0 and 10. 0 basically is no compression, "
    "1 produces shape arrays in a simple fashion. 2 and higher compression levels will use a more elaborate "
    "algorithm to find shape arrays which uses 2nd and futher neighbor distances. The higher the level, the "
    "higher the memory requirements and run times.\n"
    "Setting this property clears all format specific options for other formats such as GDS.\n"
  ) +
  gsi::method_ext ("oasis_compression_level", &get_oasis_compression,
    "@brief Get the OASIS compression level\n"
    "See \\oasis_compression_level= method for a description of the OASIS compression level."
  ),
  ""
);

#endif

}
